# -*- coding: utf-8 -*-
# Generated by Django 1.11.23 on 2019-08-29 20:03
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('leprikon', '0028_agegroups'),
    ]

    operations = [
        migrations.AlterField(
            model_name='subjectpayment',
            name='related_payment',
            field=models.OneToOneField(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, related_name='related_payments', to='leprikon.SubjectPayment', verbose_name='related payment'),
        ),
    ]
