<div class="form-group {{ item.css_classes }}">
    <label for="{{ item.id_for_label }}" class="control-label">
        {{ item.label }}
    </label>
    {{ item }}
    {% for error in item.errors %}
        <div>{{ error }}</div>
    {% endfor %}

    {% if item.help_text %}
        <span class="help-block">{{ item.help_text | safe }}</span>
    {% endif %}
</div>
